@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,15 +15,15 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Configuration storage and config-related utilities."""
 
 import copy
 import contextlib
 import functools
-import typing
-from typing import Any
+from typing import (TYPE_CHECKING, Any, Callable, Dict, Iterator, List, Mapping,
+                    MutableMapping, MutableSequence, Optional, Tuple, cast)
 
 from PyQt5.QtCore import pyqtSignal, QObject, QUrl
 
@@ -32,16 +32,15 @@ from qutebrowser.utils import utils, log, urlmatch
 from qutebrowser.misc import objects
 from qutebrowser.keyinput import keyutils
 
-if typing.TYPE_CHECKING:
-    from typing import Tuple, MutableMapping
+if TYPE_CHECKING:
     from qutebrowser.config import configcache, configfiles
     from qutebrowser.misc import savemanager
 
 # An easy way to access the config from other code via config.val.foo
-val = typing.cast('ConfigContainer', None)
-instance = typing.cast('Config', None)
-key_instance = typing.cast('KeyConfig', None)
-cache = typing.cast('configcache.ConfigCache', None)
+val = cast('ConfigContainer', None)
+instance = cast('Config', None)
+key_instance = cast('KeyConfig', None)
+cache = cast('configcache.ConfigCache', None)
 
 # Keeping track of all change filters to validate them later.
 change_filters = []
@@ -84,7 +83,7 @@ class change_filter:  # noqa: N801,N806 pylint: disable=invalid-name
                 not configdata.is_valid_prefix(self._option)):
             raise configexc.NoOptionError(self._option)
 
-    def check_match(self, option: typing.Optional[str]) -> bool:
+    def check_match(self, option: Optional[str]) -> bool:
         """Check if the given option matches the filter."""
         if option is None:
             # Called directly, not from a config change event.
@@ -97,7 +96,7 @@ class change_filter:  # noqa: N801,N806 pylint: disable=invalid-name
         else:
             return False
 
-    def __call__(self, func: typing.Callable) -> typing.Callable:
+    def __call__(self, func: Callable) -> Callable:
         """Filter calls to the decorated function.
 
         Gets called when a function should be decorated.
@@ -115,7 +114,7 @@ class change_filter:  # noqa: N801,N806 pylint: disable=invalid-name
         """
         if self._function:
             @functools.wraps(func)
-            def func_wrapper(option: str = None) -> typing.Any:
+            def func_wrapper(option: str = None) -> Any:
                 """Call the underlying function."""
                 if self.check_match(option):
                     return func()
@@ -123,8 +122,7 @@ class change_filter:  # noqa: N801,N806 pylint: disable=invalid-name
             return func_wrapper
         else:
             @functools.wraps(func)
-            def meth_wrapper(wrapper_self: typing.Any,
-                             option: str = None) -> typing.Any:
+            def meth_wrapper(wrapper_self: Any, option: str = None) -> Any:
                 """Call the underlying function."""
                 if self.check_match(option):
                     return func(wrapper_self)
@@ -142,7 +140,7 @@ class KeyConfig:
         _config: The Config object to be used.
     """
 
-    _ReverseBindings = typing.Dict[str, typing.MutableSequence[str]]
+    _ReverseBindings = Dict[str, MutableSequence[str]]
 
     def __init__(self, config: 'Config') -> None:
         self._config = config
@@ -154,10 +152,7 @@ class KeyConfig:
         if mode not in configdata.DATA['bindings.default'].default:
             raise configexc.KeybindingError("Invalid mode {}!".format(mode))
 
-    def get_bindings_for(
-            self,
-            mode: str
-    ) -> typing.Dict[keyutils.KeySequence, str]:
+    def get_bindings_for(self, mode: str) -> Dict[keyutils.KeySequence, str]:
         """Get the combined bindings for the given mode."""
         bindings = dict(val.bindings.default[mode])
         for key, binding in val.bindings.commands[mode].items():
@@ -169,7 +164,7 @@ class KeyConfig:
 
     def get_reverse_bindings_for(self, mode: str) -> '_ReverseBindings':
         """Get a dict of commands to a list of bindings for the mode."""
-        cmd_to_keys = {}  # type: KeyConfig._ReverseBindings
+        cmd_to_keys: KeyConfig._ReverseBindings = {}
         bindings = self.get_bindings_for(mode)
         for seq, full_cmd in sorted(bindings.items()):
             for cmd in full_cmd.split(';;'):
@@ -185,7 +180,7 @@ class KeyConfig:
     def get_command(self,
                     key: keyutils.KeySequence,
                     mode: str,
-                    default: bool = False) -> typing.Optional[str]:
+                    default: bool = False) -> Optional[str]:
         """Get the command for a given key (or None)."""
         self._validate(key, mode)
         if default:
@@ -206,7 +201,7 @@ class KeyConfig:
                 'mode'.format(key, mode))
 
         self._validate(key, mode)
-        log.keyboard.vdebug(  # type: ignore
+        log.keyboard.vdebug(  # type: ignore[attr-defined]
             "Adding binding {} -> {} in mode {}.".format(key, command, mode))
 
         bindings = self._config.get_mutable_obj('bindings.commands')
@@ -278,19 +273,20 @@ class Config(QObject):
                  yaml_config: 'configfiles.YamlConfig',
                  parent: QObject = None) -> None:
         super().__init__(parent)
-        self._mutables = {}  # type: MutableMapping[str, Tuple[Any, Any]]
+        self._mutables: MutableMapping[str, Tuple[Any, Any]] = {}
         self._yaml = yaml_config
         self._init_values()
         self.yaml_loaded = False
         self.config_py_loaded = False
+        self.warn_autoconfig = True
 
     def _init_values(self) -> None:
         """Populate the self._values dict."""
-        self._values = {}  # type: typing.Mapping
+        self._values: Mapping = {}
         for name, opt in configdata.DATA.items():
             self._values[name] = configutils.Values(opt)
 
-    def __iter__(self) -> typing.Iterator[configutils.Values]:
+    def __iter__(self) -> Iterator[configutils.Values]:
         """Iterate over configutils.Values items."""
         yield from self._values.values()
 
@@ -391,7 +387,7 @@ class Config(QObject):
 
     def get_obj_for_pattern(
             self, name: str, *,
-            pattern: typing.Optional[urlmatch.UrlPattern]
+            pattern: Optional[urlmatch.UrlPattern]
     ) -> Any:
         """Get the given setting as object (for YAML/config.py).
 
@@ -525,7 +521,7 @@ class Config(QObject):
         Return:
             The changed config part as string.
         """
-        lines = []  # type: typing.List[str]
+        lines: List[str] = []
         for values in sorted(self, key=lambda v: v.opt.name):
             lines += values.dump()
 
@@ -564,7 +560,7 @@ class ConfigContainer:
                               pattern=self._pattern)
 
     @contextlib.contextmanager
-    def _handle_error(self, action: str, name: str) -> typing.Iterator[None]:
+    def _handle_error(self, action: str, name: str) -> Iterator[None]:
         try:
             yield
         except configexc.Error as e:
@@ -573,6 +569,15 @@ class ConfigContainer:
             text = "While {} '{}'".format(action, name)
             self._configapi.errors.append(configexc.ConfigErrorDesc(text, e))
 
+    def _with_prefix(self, prefix: str) -> 'ConfigContainer':
+        """Get a new ConfigContainer for the given prefix."""
+        return ConfigContainer(
+            config=self._config,
+            configapi=self._configapi,
+            pattern=self._pattern,
+            prefix=prefix,
+        )
+
     def __getattr__(self, attr: str) -> Any:
         """Get an option or a new ConfigContainer with the added prefix.
 
@@ -587,9 +592,7 @@ class ConfigContainer:
 
         name = self._join(attr)
         if configdata.is_valid_prefix(name):
-            return ConfigContainer(config=self._config,
-                                   configapi=self._configapi,
-                                   prefix=name, pattern=self._pattern)
+            return self._with_prefix(name)
 
         with self._handle_error('getting', name):
             if self._configapi is None:
@@ -600,6 +603,13 @@ class ConfigContainer:
                 return self._config.get_mutable_obj(
                     name, pattern=self._pattern)
 
+        # If we arrived here, there was an error while getting the config option. Most
+        # likely, someone did something like "c.content.host_blocking.lists" but
+        # "c.content.host_blocking" doesn't actually exist. To avoid an AttributeError
+        # which leads to a confusing error message, return another ConfigContainer so
+        # that the chain can keep going.
+        return self._with_prefix(name)  # type: ignore[unreachable]
+
     def __setattr__(self, attr: str, value: Any) -> None:
         """Set the given option in the config."""
         if attr.startswith('_'):
@@ -614,5 +624,4 @@ class ConfigContainer:
         """Get the prefix joined with the given attribute."""
         if self._prefix:
             return '{}.{}'.format(self._prefix, attr)
-        else:
-            return attr
+        return attr
