@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Tests for qutebrowser.utils.utils."""
 
@@ -25,15 +25,12 @@ import os.path
 import io
 import logging
 import functools
-import socket
 import re
 import shlex
 import math
 
-import pkg_resources
-import attr
 from PyQt5.QtCore import QUrl
-from PyQt5.QtGui import QColor, QClipboard
+from PyQt5.QtGui import QClipboard
 import pytest
 import hypothesis
 from hypothesis import strategies
@@ -41,22 +38,12 @@ import yaml
 
 import qutebrowser
 import qutebrowser.utils  # for test_qualname
-from qutebrowser.utils import utils, qtutils, version, usertypes
+from qutebrowser.utils import utils, version, usertypes
 
 
 ELLIPSIS = '\u2026'
 
 
-class Color(QColor):
-
-    """A QColor with a nicer repr()."""
-
-    def __repr__(self):
-        return utils.get_repr(self, constructor=True, red=self.red(),
-                              green=self.green(), blue=self.blue(),
-                              alpha=self.alpha())
-
-
 class TestCompactText:
 
     """Test compact_text."""
@@ -142,119 +129,16 @@ class TestReadFile:
                                           'html/error.html'])
     def test_read_cached_file(self, mocker, filename):
         utils.preload_resources()
-        m = mocker.patch('pkg_resources.resource_string')
+        m = mocker.patch('qutebrowser.utils.utils.importlib_resources.files')
         utils.read_file(filename)
         m.assert_not_called()
 
     def test_readfile_binary(self):
         """Read a test file in binary mode."""
-        content = utils.read_file(os.path.join('utils', 'testfile'),
-                                  binary=True)
+        content = utils.read_file_binary(os.path.join('utils', 'testfile'))
         assert content.splitlines()[0] == b"Hello World!"
 
 
-@pytest.mark.usefixtures('freezer')
-def test_resource_filename():
-    """Read a test file."""
-    filename = utils.resource_filename(os.path.join('utils', 'testfile'))
-    with open(filename, 'r', encoding='utf-8') as f:
-        assert f.read().splitlines()[0] == "Hello World!"
-
-
-class TestInterpolateColor:
-
-    """Tests for interpolate_color.
-
-    Attributes:
-        white: The Color white as a valid Color for tests.
-        white: The Color black as a valid Color for tests.
-    """
-
-    @attr.s
-    class Colors:
-
-        white = attr.ib()
-        black = attr.ib()
-
-    @pytest.fixture
-    def colors(self):
-        """Example colors to be used."""
-        return self.Colors(Color('white'), Color('black'))
-
-    def test_invalid_start(self, colors):
-        """Test an invalid start color."""
-        with pytest.raises(qtutils.QtValueError):
-            utils.interpolate_color(Color(), colors.white, 0)
-
-    def test_invalid_end(self, colors):
-        """Test an invalid end color."""
-        with pytest.raises(qtutils.QtValueError):
-            utils.interpolate_color(colors.white, Color(), 0)
-
-    @pytest.mark.parametrize('perc', [-1, 101])
-    def test_invalid_percentage(self, colors, perc):
-        """Test an invalid percentage."""
-        with pytest.raises(ValueError):
-            utils.interpolate_color(colors.white, colors.white, perc)
-
-    def test_invalid_colorspace(self, colors):
-        """Test an invalid colorspace."""
-        with pytest.raises(ValueError):
-            utils.interpolate_color(colors.white, colors.black, 10,
-                                    QColor.Cmyk)
-
-    @pytest.mark.parametrize('colorspace', [QColor.Rgb, QColor.Hsv,
-                                            QColor.Hsl])
-    def test_0_100(self, colors, colorspace):
-        """Test 0% and 100% in different colorspaces."""
-        white = utils.interpolate_color(colors.white, colors.black, 0,
-                                        colorspace)
-        black = utils.interpolate_color(colors.white, colors.black, 100,
-                                        colorspace)
-        assert Color(white) == colors.white
-        assert Color(black) == colors.black
-
-    def test_interpolation_rgb(self):
-        """Test an interpolation in the RGB colorspace."""
-        color = utils.interpolate_color(Color(0, 40, 100), Color(0, 20, 200),
-                                        50, QColor.Rgb)
-        assert Color(color) == Color(0, 30, 150)
-
-    def test_interpolation_hsv(self):
-        """Test an interpolation in the HSV colorspace."""
-        start = Color()
-        stop = Color()
-        start.setHsv(0, 40, 100)
-        stop.setHsv(0, 20, 200)
-        color = utils.interpolate_color(start, stop, 50, QColor.Hsv)
-        expected = Color()
-        expected.setHsv(0, 30, 150)
-        assert Color(color) == expected
-
-    def test_interpolation_hsl(self):
-        """Test an interpolation in the HSL colorspace."""
-        start = Color()
-        stop = Color()
-        start.setHsl(0, 40, 100)
-        stop.setHsl(0, 20, 200)
-        color = utils.interpolate_color(start, stop, 50, QColor.Hsl)
-        expected = Color()
-        expected.setHsl(0, 30, 150)
-        assert Color(color) == expected
-
-    @pytest.mark.parametrize('percentage, expected', [
-        (0, (0, 0, 0)),
-        (99, (0, 0, 0)),
-        (100, (255, 255, 255)),
-    ])
-    def test_interpolation_none(self, percentage, expected):
-        """Test an interpolation with a gradient turned off."""
-        color = utils.interpolate_color(Color(0, 0, 0), Color(255, 255, 255),
-                                        percentage, None)
-        assert isinstance(color, QColor)
-        assert Color(color) == Color(*expected)
-
-
 @pytest.mark.parametrize('seconds, out', [
     (-1, '-0:01'),
     (0, '0:00'),
@@ -561,8 +445,12 @@ class TestIsEnum:
 
     def test_enum(self):
         """Test is_enum with an enum."""
-        e = enum.Enum('Foo', 'bar, baz')
-        assert utils.is_enum(e)
+        class Foo(enum.Enum):
+
+            bar = enum.auto()
+            baz = enum.auto()
+
+        assert utils.is_enum(Foo)
 
     def test_class(self):
         """Test is_enum with a non-enum class."""
@@ -746,13 +634,6 @@ class TestGetSetClipboard:
             utils.get_clipboard(fallback=True)
 
 
-def test_random_port():
-    port = utils.random_port()
-    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    sock.bind(('localhost', port))
-    sock.close()
-
-
 class TestOpenFile:
 
     @pytest.mark.not_frozen
@@ -801,7 +682,7 @@ class TestOpenFile:
         info = version.DistributionInfo(
             id='org.kde.Platform',
             parsed=version.Distribution.kde_flatpak,
-            version=pkg_resources.parse_version('5.12'),
+            version=utils.parse_version('5.12'),
             pretty='Unknown')
         monkeypatch.setattr(version, 'distribution',
                             lambda: info)
@@ -928,3 +809,101 @@ def test_libgl_workaround(monkeypatch, skip):
     if skip:
         monkeypatch.setenv('QUTE_SKIP_LIBGL_WORKAROUND', '1')
     utils.libgl_workaround()  # Just make sure it doesn't crash.
+
+
+@pytest.mark.parametrize('duration, out', [
+    ("0", 0),
+    ("0s", 0),
+    ("0.5s", 500),
+    ("59s", 59000),
+    ("60", 60),
+    ("60.4s", 60400),
+    ("1m1s", 61000),
+    ("1.5m", 90000),
+    ("1m", 60000),
+    ("1h", 3_600_000),
+    ("0.5h", 1_800_000),
+    ("1h1s", 3_601_000),
+    ("1h 1s", 3_601_000),
+    ("1h1m", 3_660_000),
+    ("1h1m1s", 3_661_000),
+    ("1h1m10s", 3_670_000),
+    ("10h1m10s", 36_070_000),
+])
+def test_parse_duration(duration, out):
+    assert utils.parse_duration(duration) == out
+
+
+@pytest.mark.parametrize('duration', [
+    "-1s",  # No sense to wait for negative seconds
+    "-1",
+    "34ss",
+    "",
+    "h",
+    "1.s",
+    "1.1.1s",
+    ".1s",
+    ".s",
+    "10e5s",
+    "5s10m",
+])
+def test_parse_duration_invalid(duration):
+    with pytest.raises(ValueError, match='Invalid duration'):
+        utils.parse_duration(duration)
+
+
+@hypothesis.given(strategies.text())
+def test_parse_duration_hypothesis(duration):
+    try:
+        utils.parse_duration(duration)
+    except ValueError:
+        pass
+
+
+@pytest.mark.parametrize('mimetype, extension', [
+    ('application/pdf', '.pdf'),  # handled by Python
+    ('text/plain', '.txt'),  # wrong in Python 3.6, overridden
+    ('application/manifest+json', '.webmanifest'),  # newer
+    ('text/xul', '.xul'),  # strict=False
+    ('doesnot/exist', None),
+])
+def test_mimetype_extension(mimetype, extension):
+    assert utils.mimetype_extension(mimetype) == extension
+
+
+class TestCleanupFileContext:
+
+    def test_no_file(self, tmp_path, caplog):
+        tmpfile = tmp_path / 'tmp.txt'
+        with caplog.at_level(logging.ERROR, 'misc'):
+            with utils.cleanup_file(tmpfile):
+                pass
+        assert len(caplog.messages) == 1
+        assert caplog.messages[0].startswith("Failed to delete tempfile")
+        assert not tmpfile.exists()
+
+    def test_no_error(self, tmp_path):
+        tmpfile = tmp_path / 'tmp.txt'
+        with tmpfile.open('w'):
+            pass
+        with utils.cleanup_file(tmpfile):
+            pass
+        assert not tmpfile.exists()
+
+    def test_error(self, tmp_path):
+        tmpfile = tmp_path / 'tmp.txt'
+        with tmpfile.open('w'):
+            pass
+        with pytest.raises(RuntimeError):
+            with utils.cleanup_file(tmpfile):
+                raise RuntimeError
+        assert not tmpfile.exists()
+
+    def test_directory(self, tmp_path, caplog):
+        assert tmp_path.is_dir()
+        # removal of file fails since it's a directory
+        with caplog.at_level(logging.ERROR, 'misc'):
+            with utils.cleanup_file(tmp_path):
+                pass
+        assert len(caplog.messages) == 1
+        assert caplog.messages[0].startswith("Failed to delete tempfile")
